<?php

namespace Litecms\Trader\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Trader\Http\Requests\TraderRequest;
use Litecms\Trader\Interfaces\TraderRepositoryInterface;
use Litecms\Trader\Models\Trader;

/**
 * Resource controller class for trader.
 */
class TraderResourceController extends BaseController
{

    /**
     * Initialize trader resource controller.
     *
     * @param type TraderRepositoryInterface $trader
     *
     * @return null
     */
    public function __construct(TraderRepositoryInterface $trader)
    {
        parent::__construct();
        $this->repository = $trader;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Trader\Repositories\Criteria\TraderResourceCriteria::class);
    }

    /**
     * Display a list of trader.
     *
     * @return Response
     */
    public function index(TraderRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Trader\Repositories\Presenter\TraderPresenter::class)
                ->$function();
        }

        $traders = $this->repository->paginate();

        return $this->response->title(trans('trader::trader.names'))
            ->view('trader::trader.index', true)
            ->data(compact('traders', 'view'))
            ->output();
    }

    /**
     * Display trader.
     *
     * @param Request $request
     * @param Model   $trader
     *
     * @return Response
     */
    public function show(TraderRequest $request, Trader $trader)
    {

        if ($trader->exists) {
            $view = 'trader::trader.show';
        } else {
            $view = 'trader::trader.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('trader::trader.name'))
            ->data(compact('trader'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new trader.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TraderRequest $request)
    {

        $trader = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('trader::trader.name')) 
            ->view('trader::trader.create', true) 
            ->data(compact('trader'))
            ->output();
    }

    /**
     * Create new trader.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TraderRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $trader                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('trader::trader.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('trader/trader/' . $trader->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/trader/trader'))
                ->redirect();
        }

    }

    /**
     * Show trader for editing.
     *
     * @param Request $request
     * @param Model   $trader
     *
     * @return Response
     */
    public function edit(TraderRequest $request, Trader $trader)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('trader::trader.name'))
            ->view('trader::trader.edit', true)
            ->data(compact('trader'))
            ->output();
    }

    /**
     * Update the trader.
     *
     * @param Request $request
     * @param Model   $trader
     *
     * @return Response
     */
    public function update(TraderRequest $request, Trader $trader)
    {
        try {
            $attributes = $request->all();

            $trader->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('trader::trader.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('trader/trader/' . $trader->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('trader/trader/' . $trader->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the trader.
     *
     * @param Model   $trader
     *
     * @return Response
     */
    public function destroy(TraderRequest $request, Trader $trader)
    {
        try {

            $trader->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('trader::trader.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('trader/trader/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('trader/trader/' . $trader->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple trader.
     *
     * @param Model   $trader
     *
     * @return Response
     */
    public function delete(TraderRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('trader::trader.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('trader/trader'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/trader/trader'))
                ->redirect();
        }

    }

    /**
     * Restore deleted traders.
     *
     * @param Model   $trader
     *
     * @return Response
     */
    public function restore(TraderRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('trader::trader.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/trader/trader'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/trader/trader/'))
                ->redirect();
        }

    }

}
